{ Copyright (c) 1999, 2000 by Mandys Tomas - MandySoft }
{ email: tomas.mandys@2p.cz }
{ URL: http://www.2p.cz }

unit Dbutils;

{.$LONGSTRINGS OFF} { short strings }

interface
uses
  Classes, DB, DBTables, SysUtils, dbiTypes;

procedure CompareKeyRecords(const DBName, TblName1, TblName2, KeyFld1, KeyFld2, FlagFld1, FlagFld2: string);
function dbiSetDelete(Table: TTable; OnOff: Boolean): DBIResult;
function DBAnsiUpper(Table: TTable; const S: string): ShortString;
function DBAnsiLower(Table: TTable; const S: string): ShortString;
function DBAnsiCmp(Table: TTable; S1, S2: string): Integer;

function GetTempFileName(const S: string): string;

procedure CreateTable(const ADatabaseName, ATableName: string; const AFieldDefs: array of PChar);
procedure CreateIndex(const ADatabaseName, ATableName, AIndexName, AIndexFlds: string);
procedure ExportTable(DataSet: TDataSet; const FieldNames: array of PChar;
  const OutputFileName: string; CreateFile: Boolean; const Delimiter, Separator: string);
procedure ExportAllFields(DataSet: TDataSet; AVisible, ACalculated: Byte;
  const OutputFileName: string; CreateFile: Boolean; const Delimiter, Separator: string);
function TableToStringList(const ADatabaseName, ATableName, AIndexName, AFormat: string; const Arr: array of string): TStrings;
procedure CopyStream(SourceFld, DestFld: TBlobField);
function GetMemoAsString(SourceFld: TBlobField): string;
function GetAliasPath(const Alias: string): string;
procedure AdjustPath(var S: string);
function GetOlePath(S: string): string;
function SetOlePath(S: string): string;
procedure PrintStream(St: TStream);
function GetUserName: string;
procedure MakePermanent(hCursor: hDBICur; aName: string);
function _GetRecordCount(T: TTable): Longint;

implementation
uses
  AuxStr, GaugeFrm, dbiProcs, dbiErrs, Dialogs, WinTypes, WinProcs, Printers, CzStr;

procedure AdjustPath;
begin
  if (S <> '') and not (S[Length(S)] in ['\',':']) then
    S:= S+'\';
end;

procedure CompareKeyRecords;
{ porovnava exestenci zaznamu ve dvou tabulkach }
{ v prvni oznaci '-' recordy ktere neexistuji v druhe, v ve druhe '=' ktere jsou v obou tabulkach }
var
  G: TGaugeForm;
  T: TTable;
begin
  T:= TTable.create(nil);
  with TTable.Create(nil) do
  try
    DatabaseName:= DBName;
    T.DatabaseName:= DBName;
    Tablename:= TblName1;
    T.Tablename:= TblName2;
    T.IndexName:= KeyFld2;
    Open;
    T.Open;
    G:= ShowGauge(0, RecordCount);
    try
      while not EOF do
      begin
        if T.FindKey([Fieldbyname(KeyFld1).AsString]) then
        begin
          if FlagFld2 <> '' then
          begin
            T.Edit;
            T.FieldByName(FlagFld2).AsString:= '=';
            T.Post;
          end
        end
      else
        begin
          if FlagFld1 <> '' then
          begin
            Edit;
            FieldByName(FlagFld1).AsString:= '-';
            Post;
          end;
        end;
        Next;
        G.Gauge.Progress:= G.Gauge.Progress+1;
        {if G.Gauge.Progress > 10 then exit;}
      end;
    finally
      G.Free;
    end;
  finally
    Free;
    T.Free;
  end;
end;

function dbiSetDelete;
begin
  try
    Table.DisableControls;
    Result := DbiSetProp(hDBIObj(Table.Handle), curSOFTDELETEON, LongInt(OnOff));
  finally
    Table.Refresh;
    Table.EnableControls;
  end;
end;

function DBAnsiUpper;
var
  S2: ShortString;
  P: Pointer;
  B: Bool;
begin
  Result:= S;
  dbiGetLdObj(Table.Handle, P);
  dbiAnsiToNative(P, @S2[1], @Result[1], Length(Result), B);

  SetLength(S2, Length(Result));
  S2:= S2+#0;
  OsLdStrToUpper(Table.Locale, @S2[1]);
  dbiNativeToAnsi(P, @Result[1], @S2[1], Length(Result), B);
end;

function DBAnsiLower;
var
  S2: ShortString;
  P: Pointer;
  B: Bool;
begin
  Result:= S;
  dbiGetLdObj(Table.Handle, P);
  dbiAnsiToNative(P, @S2[1], @Result[1], Length(Result), B);

  SetLength(S2, Length(Result));
  S2:= S2+#0;
  OsLdStrToLower(Table.Locale, @S2[1]);
  dbiNativeToAnsi(P, @Result[1], @S2[1], Length(Result), B);
end;

function DBAnsiCmp;
{var
  P: Pointer;
  nS1, nS2: string;
  B: Bool;}

  procedure StripSamohl(var S: string);
  var
    S1: string;
    I: Integer;
  begin
    S1:= S;
    CzSwitchTo(czWin1250, czAscii, S1);
    for I:= 1 to Length(S) do
      if Upcase(S1[I]) in ['A','E','I','O','U','Y'] then  { dlouhe samohlasky --> kratke }
        S[I]:= S1[I];
  end;
begin
{  Result:= AnsiCompareStr(S1, S2);  { porovnava ale A <  < B }
{  dbiGetLdObj(Table.Handle, P);
  dbiAnsiToNative(P, @nS1[1], @S1[1], Length(S1), B);
  dbiAnsiToNative(P, @nS2[1], @S2[1], Length(S2), B);
  nS1[0]:= S1[0]; nS1:= nS1+#0;
  nS2[0]:= S2[0]; nS2:= nS2+#0;
  Result:= OsLdStrCmpI(Table.Locale, @nS1[1], @nS2[1]);  { naporovnava narodni znaky }
  { potreba a =  < b }
  StripSamohl(S1);
  StripSamohl(S2);
  Result:= AnsiCompareStr(S1, S2);  { porovnani bez dlouhych samohl }
end;

function GetTempFileName;
begin
  Result:= Format('%s%d', [Copy(S, 1, 3), Word(GetTickCount)]);
end;

procedure CreateTable;
var
  S: string;
  I: Integer;
begin
  with TQuery.Create(nil) do
  try
    DatabaseName:= ADatabaseName;
    try   
      SQL.Add(Format('DROP TABLE "%s"', [ATableName]));
      ExecSQL;
    except
    end;
    SQL.Clear;
    SQL.Add(Format('CREATE TABLE "%s" (', [ATableName]));
    S:= '';
    for I:= Low(AFieldDefs) to High(AFieldDefs) do
      if (AFieldDefs[I] <> nil) and (StrPas(AFieldDefs[I]) <> '') then
        begin
          SQL.Add(S+StrPas(AFieldDefs[I]));
          S:= ','
        end;
    SQL.Add(')');
    ExecSQL;
  finally
    Free;
  end;
end;

procedure CreateIndex;
begin
  with TQuery.Create(nil) do
  try
    DatabaseName:= ADatabaseName;
    try
      SQL.Add(Format('CREATE INDEX %s ON "%s" (%s)', [AIndexName, ATableName, AIndexFlds]));
      ExecSQL;
    except
    end;
  finally
    Free;
  end;
end;

procedure ExportTable;
var
  S: string;
  Ext: string[4];
  FM: Word;
  B: TBookmark;
  I, J: Integer;
  G: TGaugeForm;
  F: TField;
  T: TTable;
  St: TStream;
  A: array[1..20] of PChar;
const
  CRLF = #13#10;
begin
  Ext:= UpString(ExtractFileExt(OutputFileName));
  B:= DataSet.GetBookmark;
  try
    G:= ShowGauge(0, DataSet.RecordCount);
    try
      if (Ext = '.DB') or (Ext = '.DBF') then
        begin
          if CreateFile then
          begin
            FillChar(A, SizeOf(A), 0);
            J:= Low(A);
            for I:= Low(FieldNames) to High(FieldNames) do
              if (FieldNames[I] <> nil) and (StrPas(FieldNames[I]) <> '') then
              begin
                F:= DataSet.FieldByName(StrPas(FieldNames[I]));
                S:= F.FieldName + ' ';
                if F is TStringField then S:= Format('%s CHAR(%d)'#0, [S, F.Size]) else
                if (F is TIntegerField) or (F is TFloatField) { vetsinou float je ve skutecnosti integer } then
                  S:= Format('%s INTEGER'#0, [S]) else
                if F is TSmallIntField then S:= Format('%s SMALLINT'#0, [S]) else
                if F is TFloatField then S:= Format('%s NUMERIC(%d,2)'#0, [S, (F as TFloatField).Precision]) else
                if F is TBooleanField then S:= Format('%s BOOLEAN'#0, [S]);
                A[J]:= StrNew(@S[1]);
                Inc(J);
              end;
            CreateTable('', OutputFileName, A);
            for I:= Low(A) to High(A) do
              if A[I] <> nil then
                StrDispose(A[I]);
          end;
          DataSet.First;
          T:= TTable.Create(nil);
          try
            T.TableName:= OutputFileName;
            T.Open;
            while not DataSet.EOF do
            begin
              T.Insert;
              for I:= Low(FieldNames) to High(FieldNames) do
                if (FieldNames[I] <> nil) and (StrPas(FieldNames[I]) <> '') then
                begin
                  F:= T.FindField(StrPas(FieldNames[I]));
                  if F <> nil then
                    F.AsString:= DataSet.FieldByName(F.FieldName).AsString;
                end;
              T.Post;
              DataSet.Next;
              G.Gauge.Progress:= G.Gauge.Progress+1;
            end;
          finally
            T.Free;
          end;
        end
      else
        begin
          if OutputFileName = '<PRN>' then
            St:= TMemoryStream.Create
          else
            begin
              if CreateFile then FM:= fmCreate
                            else FM:= fmOpenWrite;
              St:= TFileStream.Create(OutputFileName, FM);
            end;
          try
            St.Position:= St.Size;
            DataSet.First;
            while not DataSet.EOF do
            begin
              S:= '';
              for I:= Low(FieldNames) to High(FieldNames) do
                if (FieldNames[I] <> nil) and (StrPas(FieldNames[I]) <> '') then
                begin
                  S:= Format('%s%s%s%s', [S, Delimiter, DataSet.FieldByName(StrPas(FieldNames[I])).AsString, Delimiter]);
                  St.WriteBuffer(S[1], Length(S));
                  S:= Separator;
                end;
              S:= CRLF;
              St.WriteBuffer(S[1], Length(S));
              DataSet.Next;
              G.Gauge.Progress:= G.Gauge.Progress+1;
            end;
          finally
            if OutputFileName = '<PRN>' then
              PrintStream(St);
            St.Free;
          end;
        end;
    finally
      G.Free;
    end;
  finally
    DataSet.GotoBookmark(B);
    DataSet.FreeBookmark(B);
  end;
end;

procedure ExportAllFields;
var
  A: array[1..255] of PChar;
  I, J: Integer;
  S: string;
begin
  FillChar(A, SizeOf(A), 0);
  J:= Low(A);
  for I:= 0 to DataSet.FieldCount-1 do
  begin
    if (ACalculated <= 1) and ((DataSet.Fields[I].FieldKind in [fkLookup, fkCalculated]) <> Boolean(ACalculated)) then
      Continue;
    if (AVisible <= 1) and (DataSet.Fields[I].Visible <> Boolean(AVisible)) then
      Continue;
    S:= Dataset.Fields[I].FieldName+#0;
    A[J]:= StrNew(@S[1]);
    Inc(J);
  end;
  if J > Low(A) then
    ExportTable(DataSet, A, OutputFileName, CreateFile, Delimiter, Separator);
  for I:= Low(A) to J-1 do
    StrDispose(A[I]);
end;

function TableToStringList;
var
  T: TTable;
  A: array[0..20] of TVarRec;
  I: Integer;
begin
  Result:= TStringList.Create;
  T:= TTable.Create(nil);
  try
    T.DatabaseName:= ADatabaseName;
    T.TableName:= ATableName;
    T.IndexName:= AIndexName;
    T.ReadOnly:= True;
    T.Open;
    while not T.EOF do
    begin
      FillChar(A, SizeOf(A), 0);
      for I:= Low(Arr) to High(Arr) do
      begin
        A[I-Low(Arr)].vType:= vtString;
        A[I-Low(Arr)].vString:= Pointer(NewStr(T.FieldByName(Arr[I]).AsString));
      end;
      Result.Add(Format(AFormat, A));
      for I:= Low(A) to High(A) do
        DisposeStr(PString(A[I].vString));
      T.Next;
    end;
  finally
    T.Free;
  end;
end;

procedure CopyStream;
var
  S1, S2: TStream;
  J: Integer;
  Buff: array[1..1024] of Char;
begin
  S1:= TBlobStream.Create(SourceFld, bmRead);
  try
    if S1.Size <> 0 then
    begin
      S2:= TBlobStream.Create(DestFld, bmWrite);
      try
        S2.Position:= S2.Size;
        while S1.Position < S1.Size do
        begin
          J:= S1.Size - S1.Position;
          if J > SizeOf(Buff) then
            J:= SizeOf(Buff);
          S1.ReadBuffer(Buff, J);
          S2.WriteBuffer(Buff, J);
        end;
      finally
        S2.Free;
      end;
    end;
  finally
    S1.Free;
  end;
end;

function GetMemoAsString(SourceFld: TBlobField): string;
var
  S1: TStream;
begin
  S1:= TBlobStream.Create(SourceFld, bmRead);
  try
    SetLength(Result, S1.Size);
    S1.ReadBuffer(Result[1], S1.Size);
  finally
    S1.Free;
  end;
end;

function GetAliasPath;
var
  S: TStrings;
begin
  S:= TStringList.Create;
  try
    Session.GetAliasParams(Alias, S);
    Result:= S.Values['PATH'];
    AdjustPath(Result);
  finally
    S.Free;
  end;
end;

function GetOlePath;
var
  St: TStrings;
  S2: string;
  I: Integer;
begin
  Result:= S;
  if S <> '' then
  begin
    if S[1] = ':' then
    begin
      Delete(S, 1, 1);
      I:= Pos(':', S);
      if I > 0 then
      begin
        St:= TStringList.Create;
        try
          try
            Session.GetAliasParams(Copy(S, 1, I-1), St);
            Delete(S, 1, I);
            if Pos('\', S) = 1 then
              Delete(S, 1, 1);
            S2:= St.Values['PATH'];
            AdjustPath(S2);
            Result:= S2+S;
          except
          { kdyz nexe. tak se nic nedeje }
          end;
        finally
          St.Free;
        end;
      end;
    end;
  end;
end;

function SetOlePath;
var
  AliasSt, St: TStrings;
  S2: string;
  I, J: Integer;
  MaxLen: Integer;
begin
  Result:= S;
  AliasSt:= TStringList.Create;
  try
    Session.GetAliasNames(AliasSt);
    MaxLen:= 0;
    for I:= 0 to AliasSt.Count-1 do
    begin
      St:= TStringList.Create;
      try
        Session.GetAliasParams(AliasSt.Strings[I], St);
        S2:= AnsiUpperCase(St.Values['PATH']);
        if S2 <> '' then
        begin
          AdjustPath(S2);
          J:= Pos(S2, S);
          if (J > 0) and (J > MaxLen) then
          begin
            MaxLen:= J;
            Delete(S, 1, Length(S2));
            Result:= ':'+AliasSt.Strings[I]+':'+S;
          end;
        end;
      finally
        St.Free;
      end;
    end;
  finally
    AliasSt.Free;
  end;
end;

procedure PrintStream;
var
  T: Text;
  C: Char;
begin
  if St.Size = 0 then Exit;
  AssignPrn(T);
  Rewrite(T);
  St.Position:= 0;
  while St.Position < St.Size do
  begin
    St.ReadBuffer(C, SizeOf(C));
    Write(T, C);
  end;
  System.CloseFile(T)
end;

function GetUserName;
var
  Buffer: array[0..255] of Char;
begin
{ Result:= GetProp(sesUserName); }
  dbiGetNetUserName(Buffer);
  Result:= StrPas(Buffer);
end;

procedure MakePermanent;
var
  Status: dbiResult;
begin
  aName:= aName+#0;
  Status:= dbiMakePermanent(hCursor, @aName[1], True);
  if Status <> 0 then
    raise EDBEngineError.Create(Status);
end;

function _GetRecordCount(T: TTable): Longint;
begin
  Check(DbiGetRecordCount(T.Handle, Result));  { metoda vraci ExactRecordCount coz nepotrebuju - pomale }
end;

end.
